@@ -9,11 +9,13 @@ use std::path::{Path as FilePath, PathBuf as FilePathBuf};
 
 use syn;
 
+use bindgen::bitflags;
 use bindgen::cargo::{Cargo, PackageRef};
+use bindgen::config::MacroExpansionConfig;
 use bindgen::error::Error;
 use bindgen::ir::{
-    AnnotationSet, Cfg, Constant, Documentation, Enum, Function, GenericParams, ItemContainer,
-    ItemMap, OpaqueItem, Path, Static, Struct, Type, Typedef, Union,
+    AnnotationSet, Cfg, Constant, Documentation, Enum, Function, GenericParams, ItemMap,
+    OpaqueItem, Path, Static, Struct, Type, Typedef, Union,
 };
 use bindgen::utilities::{SynAbiHelpers, SynItemHelpers};
 
@@ -29,11 +31,15 @@ const STD_CRATES: &'static [&'static str] = &[
 type ParseResult = Result<Parse, Error>;
 
 /// Parses a single rust source file, not following `mod` or `extern crate`.
-pub fn parse_src(src_file: &FilePath) -> ParseResult {
+pub fn parse_src(
+    src_file: &FilePath,
+    macro_expansion_config: &MacroExpansionConfig,
+) -> ParseResult {
     let mod_name = src_file.file_stem().unwrap().to_str().unwrap();
 
     let mut context = Parser {
         binding_crate_name: mod_name.to_owned(),
+        macro_expansion_config,
         lib: None,
         parse_deps: true,
         include: None,
@@ -65,6 +71,7 @@ pub fn parse_src(src_file: &FilePath) -> ParseResult {
 /// command to find the location of dependencies.
 pub(crate) fn parse_lib(
     lib: Cargo,
+    macro_expansion_config: &MacroExpansionConfig,
     parse_deps: bool,
     include: &Option<Vec<String>>,
     exclude: &[String],
@@ -75,6 +82,7 @@ pub(crate) fn parse_lib(
 ) -> ParseResult {
     let mut context = Parser {
         binding_crate_name: lib.binding_crate_name().to_owned(),
+        macro_expansion_config,
         lib: Some(lib),
         parse_deps: parse_deps,
         include: include.clone(),
@@ -96,8 +104,9 @@ pub(crate) fn parse_lib(
 }
 
 #[derive(Debug, Clone)]
-struct Parser {
+struct Parser<'a> {
     binding_crate_name: String,
+    macro_expansion_config: &'a MacroExpansionConfig,
     lib: Option<Cargo>,
     parse_deps: bool,
 
@@ -117,7 +126,7 @@ struct Parser {
     out: Parse,
 }
 
-impl Parser {
+impl<'a> Parser<'a> {
     fn should_parse_dependency(&self, pkg_name: &String) -> bool {
         if self.parsed_crates.contains(pkg_name) {
             return false;
@@ -200,9 +209,10 @@ impl Parser {
 
     fn process_expanded_mod(&mut self, pkg: &PackageRef, items: &[syn::Item]) -> Result<(), Error> {
         self.out.load_syn_crate_mod(
+            &self.macro_expansion_config,
             &self.binding_crate_name,
             &pkg.name,
-            &Cfg::join(&self.cfg_stack),
+            Cfg::join(&self.cfg_stack).as_ref(),
             items,
         );
 
@@ -210,8 +220,8 @@ impl Parser {
             if item.has_test_attr() {
                 continue;
             }
-            match item {
-                &syn::Item::Mod(ref item) => {
+            match *item {
+                syn::Item::Mod(ref item) => {
                     let cfg = Cfg::load(&item.attrs);
                     if let &Some(ref cfg) = &cfg {
                         self.cfg_stack.push(cfg.clone());
@@ -227,7 +237,7 @@ impl Parser {
                         self.cfg_stack.pop();
                     }
                 }
-                &syn::Item::ExternCrate(ref item) => {
+                syn::Item::ExternCrate(ref item) => {
                     let dep_pkg_name = item.ident.to_string();
 
                     let cfg = Cfg::load(&item.attrs);
@@ -309,9 +319,10 @@ impl Parser {
         items: &[syn::Item],
     ) -> Result<(), Error> {
         self.out.load_syn_crate_mod(
+            &self.macro_expansion_config,
             &self.binding_crate_name,
             &pkg.name,
-            &Cfg::join(&self.cfg_stack),
+            Cfg::join(&self.cfg_stack).as_ref(),
             items,
         );
 
@@ -319,8 +330,8 @@ impl Parser {
             if item.has_test_attr() {
                 continue;
             }
-            match item {
-                &syn::Item::Mod(ref item) => {
+            match *item {
+                syn::Item::Mod(ref item) => {
                     let next_mod_name = item.ident.to_string();
 
                     let cfg = Cfg::load(&item.attrs);
@@ -374,7 +385,7 @@ impl Parser {
                         self.cfg_stack.pop();
                     }
                 }
-                &syn::Item::ExternCrate(ref item) => {
+                syn::Item::ExternCrate(ref item) => {
                     let dep_pkg_name = item.ident.to_string();
 
                     let cfg = Cfg::load(&item.attrs);
@@ -485,9 +496,10 @@ impl Parse {
 
     pub fn load_syn_crate_mod(
         &mut self,
+        macro_expansion_config: &MacroExpansionConfig,
         binding_crate_name: &str,
         crate_name: &str,
-        mod_cfg: &Option<Cfg>,
+        mod_cfg: Option<&Cfg>,
         items: &[syn::Item],
     ) {
         let mut impls_with_assoc_consts = Vec::new();
@@ -530,31 +542,42 @@ impl Parse {
                         impls_with_assoc_consts.push(item_impl);
                     }
                 }
+                syn::Item::Macro(ref item) => {
+                    self.load_builtin_macro(macro_expansion_config, crate_name, mod_cfg, item)
+                }
                 _ => {}
             }
         }
 
         for item_impl in impls_with_assoc_consts {
-            let associated_constants = item_impl.items.iter().filter_map(|item| match item {
-                syn::ImplItem::Const(ref associated_constant) => Some(associated_constant),
-                _ => None,
-            });
-            self.load_syn_assoc_consts(
-                binding_crate_name,
-                crate_name,
-                mod_cfg,
-                &item_impl.self_ty,
-                associated_constants,
-            );
+            self.load_syn_assoc_consts_from_impl(crate_name, mod_cfg, item_impl)
         }
     }
 
+    fn load_syn_assoc_consts_from_impl(
+        &mut self,
+        crate_name: &str,
+        mod_cfg: Option<&Cfg>,
+        item_impl: &syn::ItemImpl,
+    ) {
+        let associated_constants = item_impl.items.iter().filter_map(|item| match item {
+            syn::ImplItem::Const(ref associated_constant) => Some(associated_constant),
+            _ => None,
+        });
+        self.load_syn_assoc_consts(
+            crate_name,
+            mod_cfg,
+            &item_impl.self_ty,
+            associated_constants,
+        );
+    }
+
     /// Enters a `extern "C" { }` declaration and loads function declarations.
     fn load_syn_foreign_mod(
         &mut self,
         binding_crate_name: &str,
         crate_name: &str,
-        mod_cfg: &Option<Cfg>,
+        mod_cfg: Option<&Cfg>,
         item: &syn::ItemForeignMod,
     ) {
         if !item.abi.is_c() {
@@ -563,8 +586,8 @@ impl Parse {
         }
 
         for foreign_item in &item.items {
-            match foreign_item {
-                &syn::ForeignItem::Fn(ref function) => {
+            match *foreign_item {
+                syn::ForeignItem::Fn(ref function) => {
                     if crate_name != binding_crate_name {
                         info!(
                             "Skip {}::{} - (fn's outside of the binding crate are not used).",
@@ -597,7 +620,7 @@ impl Parse {
         &mut self,
         binding_crate_name: &str,
         crate_name: &str,
-        mod_cfg: &Option<Cfg>,
+        mod_cfg: Option<&Cfg>,
         item: &syn::ItemFn,
     ) {
         if crate_name != binding_crate_name {
@@ -644,46 +667,11 @@ impl Parse {
         }
     }
 
-    fn is_assoc_const_of_transparent_struct(
-        &self,
-        const_item: &syn::ImplItemConst,
-    ) -> Result<bool, ()> {
-        let ty = match Type::load(&const_item.ty) {
-            Ok(Some(t)) => t,
-            _ => return Ok(false),
-        };
-        let path = match ty.get_root_path() {
-            Some(p) => p,
-            _ => return Ok(false),
-        };
-
-        match self.structs.get_items(&path) {
-            Some(items) => {
-                if items.len() != 1 {
-                    error!(
-                        "Expected one struct to match path {}, but found {}",
-                        path,
-                        items.len(),
-                    );
-                    return Err(());
-                }
-
-                Ok(if let ItemContainer::Struct(ref s) = items[0] {
-                    s.is_transparent
-                } else {
-                    false
-                })
-            }
-            _ => Ok(false),
-        }
-    }
-
     /// Loads associated `const` declarations
     fn load_syn_assoc_consts<'a, I>(
         &mut self,
-        binding_crate_name: &str,
         crate_name: &str,
-        mod_cfg: &Option<Cfg>,
+        mod_cfg: Option<&Cfg>,
         impl_ty: &syn::Type,
         items: I,
     ) where
@@ -703,35 +691,29 @@ impl Parse {
         let impl_path = ty.unwrap().get_root_path().unwrap();
 
         for item in items.into_iter() {
-            let is_assoc_const_of_transparent_struct =
-                match self.is_assoc_const_of_transparent_struct(&item) {
-                    Ok(b) => b,
-                    Err(_) => continue,
-                };
-
-            if crate_name != binding_crate_name {
-                info!(
-                    "Skip {}::{} - (const's outside of the binding crate are not used).",
-                    crate_name, &item.ident
-                );
-                continue;
-            }
-
-            let const_name = item.ident.to_string();
-
-            match Constant::load_assoc(
-                const_name,
-                item,
+            let path = Path::new(item.ident.to_string());
+            match Constant::load(
+                path,
                 mod_cfg,
-                is_assoc_const_of_transparent_struct,
-                &impl_path,
+                &item.ty,
+                &item.expr,
+                &item.attrs,
+                Some(impl_path.clone()),
             ) {
                 Ok(constant) => {
-                    info!("Take {}::{}.", crate_name, &item.ident);
-
-                    let full_name = constant.path.clone();
-                    if !self.constants.try_insert(constant) {
-                        error!("Conflicting name for constant {}", full_name);
+                    info!("Take {}::{}::{}.", crate_name, impl_path, &item.ident);
+                    let mut any = false;
+                    self.structs.for_items_mut(&impl_path, |item| {
+                        any = true;
+                        item.add_associated_constant(constant.clone());
+                    });
+                    // Handle associated constants to other item types that are
+                    // not structs like enums or such as regular constants.
+                    if !any && !self.constants.try_insert(constant) {
+                        error!(
+                            "Conflicting name for constant {}::{}::{}.",
+                            crate_name, impl_path, &item.ident,
+                        );
                     }
                 }
                 Err(msg) => {
@@ -746,7 +728,7 @@ impl Parse {
         &mut self,
         binding_crate_name: &str,
         crate_name: &str,
-        mod_cfg: &Option<Cfg>,
+        mod_cfg: Option<&Cfg>,
         item: &syn::ItemConst,
     ) {
         if crate_name != binding_crate_name {
@@ -758,8 +740,7 @@ impl Parse {
         }
 
         let path = Path::new(item.ident.to_string());
-
-        match Constant::load(path, item, mod_cfg) {
+        match Constant::load(path, mod_cfg, &item.ty, &item.expr, &item.attrs, None) {
             Ok(constant) => {
                 info!("Take {}::{}.", crate_name, &item.ident);
 
@@ -779,7 +760,7 @@ impl Parse {
         &mut self,
         binding_crate_name: &str,
         crate_name: &str,
-        mod_cfg: &Option<Cfg>,
+        mod_cfg: Option<&Cfg>,
         item: &syn::ItemStatic,
     ) {
         if crate_name != binding_crate_name {
@@ -816,11 +797,10 @@ impl Parse {
     }
 
     /// Loads a `struct` declaration
-    fn load_syn_struct(&mut self, crate_name: &str, mod_cfg: &Option<Cfg>, item: &syn::ItemStruct) {
+    fn load_syn_struct(&mut self, crate_name: &str, mod_cfg: Option<&Cfg>, item: &syn::ItemStruct) {
         match Struct::load(item, mod_cfg) {
             Ok(st) => {
                 info!("Take {}::{}.", crate_name, &item.ident);
-
                 self.structs.try_insert(st);
             }
             Err(msg) => {
@@ -834,7 +814,7 @@ impl Parse {
     }
 
     /// Loads a `union` declaration
-    fn load_syn_union(&mut self, crate_name: &str, mod_cfg: &Option<Cfg>, item: &syn::ItemUnion) {
+    fn load_syn_union(&mut self, crate_name: &str, mod_cfg: Option<&Cfg>, item: &syn::ItemUnion) {
         match Union::load(item, mod_cfg) {
             Ok(st) => {
                 info!("Take {}::{}.", crate_name, &item.ident);
@@ -852,7 +832,7 @@ impl Parse {
     }
 
     /// Loads a `enum` declaration
-    fn load_syn_enum(&mut self, crate_name: &str, mod_cfg: &Option<Cfg>, item: &syn::ItemEnum) {
+    fn load_syn_enum(&mut self, crate_name: &str, mod_cfg: Option<&Cfg>, item: &syn::ItemEnum) {
         if item.generics.lifetimes().count() > 0 {
             info!(
                 "Skip {}::{} - (has generics or lifetimes or where bounds).",
@@ -877,7 +857,7 @@ impl Parse {
     }
 
     /// Loads a `type` declaration
-    fn load_syn_ty(&mut self, crate_name: &str, mod_cfg: &Option<Cfg>, item: &syn::ItemType) {
+    fn load_syn_ty(&mut self, crate_name: &str, mod_cfg: Option<&Cfg>, item: &syn::ItemType) {
         match Typedef::load(item, mod_cfg) {
             Ok(st) => {
                 info!("Take {}::{}.", crate_name, &item.ident);
@@ -893,4 +873,36 @@ impl Parse {
             }
         }
     }
+
+    fn load_builtin_macro(
+        &mut self,
+        macro_expansion_config: &MacroExpansionConfig,
+        crate_name: &str,
+        mod_cfg: Option<&Cfg>,
+        item: &syn::ItemMacro,
+    ) {
+        let name = match item.mac.path.segments.last() {
+            Some(ref n) => n.value().ident.to_string(),
+            None => return,
+        };
+
+        if name != "bitflags" || !macro_expansion_config.bitflags {
+            return;
+        }
+
+        let bitflags = match bitflags::parse(item.mac.tts.clone()) {
+            Ok(b) => b,
+            Err(e) => {
+                warn!("Failed to parse bitflags invocation: {:?}", e);
+                return;
+            }
+        };
+
+        let (struct_, impl_) = bitflags.expand();
+        self.load_syn_struct(crate_name, mod_cfg, &struct_);
+        // We know that the expansion will only reference `struct_`, so it's
+        // fine to just do it here instead of deferring it like we do with the
+        // other calls to this function.
+        self.load_syn_assoc_consts_from_impl(crate_name, mod_cfg, &impl_);
+    }
 }
