@@ -20,5 +20,24 @@ pub enum Bar<T> {
     Bar4,
 }
 
+#[repr(u8)]
+pub enum Baz {
+    Baz1(Bar<u32>),
+    Baz2(Point<i32>),
+    Baz3,
+}
+
+#[repr(C, u8)]
+pub enum Taz {
+    Taz1(Bar<u32>),
+    Taz2(Baz),
+    Taz3,
+}
+
 #[no_mangle]
-pub extern "C" fn foo(foo: *const Foo<i32>, bar: *const Bar<i32>) {}
+pub extern "C" fn foo(
+    foo: *const Foo<i32>,
+    bar: *const Bar<i32>,
+    baz: *const Baz,
+    taz: *const Taz,
+) {}
